@@ -1,23 +1,24 @@
 /* classes: h_files */
 
-#ifndef _SCMH
-#define _SCMH
-/*	Copyright (C) 1995,1996, 2000, 2001 Free Software Foundation, Inc.
- * 
+#ifndef SCM__SCM_H
+#define SCM__SCM_H
+
+/* Copyright (C) 1995,1996,2000,2001, 2002, 2006 Free Software Foundation, Inc.
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA
  *
  * As a special exception, the Free Software Foundation gives permission
  * for additional uses of the text contained in its release of GUILE.
@@ -42,22 +43,25 @@
  * If you write modifications of your own for GUILE, it is your choice
  * whether to permit this exception to apply to your modifications.
  * If you do not wish that, delete this exception notice.  */
+
 
 
+/**********************************************************************
+ This file is Guile's central private header.
+
+ When included by other files, this file should preceed any include
+ other than __scm.h.  See __scm.h for details regarding the purpose of
+ and differences between _scm.h and __scm.h.
+ **********************************************************************/
+
+
+#if HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
+#include <errno.h>
 #include "libguile/__scm.h"
 
-/* "What's the difference between _scm.h and __scm.h?"
-
-   _scm.h is not installed; it's only visible to the libguile sources
-   themselves.
-
-   __scm.h is installed, and is #included by <libguile.h>.  If both
-   the client and libguile need some piece of information, and it
-   doesn't fit well into the header file for any particular module, it
-   should go in __scm.h.  */
-
-
-
 /* Include headers for those files central to the implementation.  The
    rest should be explicitly #included in the C files themselves.  */
 #include "libguile/error.h"	/* Everyone signals errors.  */
@@ -70,13 +74,11 @@
 #include "libguile/numbers.h"	/* Everyone deals with fixnums.  */
 #include "libguile/symbols.h"	/* For length, chars, values, miscellany.  */
 #include "libguile/boolean.h"	/* Everyone wonders about the truth.  */
-#ifdef USE_THREADS		
-#include "libguile/threads.h"	/* The cooperative thread package does
-				   switching at async ticks.  */
-#endif
+#include "libguile/threads.h"	/* You are not alone. */
 #include "libguile/snarf.h"	/* Everyone snarfs. */
 #include "libguile/variable.h"
 #include "libguile/modules.h"
+#include "libguile/inline.h"
 
 /* SCM_SYSCALL retries system calls that have been interrupted (EINTR).
    However this can be avoided if the operating system can restart
@@ -86,8 +88,11 @@
    */
 
 #ifdef HAVE_RESTARTABLE_SYSCALLS
+#if ! SCM_USE_PTHREAD_THREADS /* However, don't assume SA_RESTART 
+                                 works with pthreads... */
 #define SCM_SYSCALL(line) line
 #endif
+#endif
 
 #ifndef SCM_SYSCALL
 #ifdef vms
@@ -122,15 +127,52 @@
 
 
 #ifndef min
-#define min(A,B) ((A) <= (B) ? (A) : (B))
+#define min(A, B) ((A) <= (B) ? (A) : (B))
 #endif
 #ifndef max
-#define max(A,B) ((A) >= (B) ? (A) : (B))
+#define max(A, B) ((A) >= (B) ? (A) : (B))
 #endif
 
-#endif  /* _SCMH */
 
 
+#if HAVE_STAT64
+#define CHOOSE_LARGEFILE(foo,foo64)     foo64
+#else
+#define CHOOSE_LARGEFILE(foo,foo64)     foo
+#endif
+
+/* These names are a bit long, but they make it clear what they represent. */
+#define dirent_or_dirent64              CHOOSE_LARGEFILE(dirent,dirent64)
+#define fstat_or_fstat64                CHOOSE_LARGEFILE(fstat,fstat64)
+#define ftruncate_or_ftruncate64        CHOOSE_LARGEFILE(ftruncate,ftruncate64)
+#define lseek_or_lseek64                CHOOSE_LARGEFILE(lseek,lseek64)
+#define lstat_or_lstat64                CHOOSE_LARGEFILE(lstat,lstat64)
+#define off_t_or_off64_t                CHOOSE_LARGEFILE(off_t,off64_t)
+#define open_or_open64                  CHOOSE_LARGEFILE(open,open64)
+#define readdir_or_readdir64            CHOOSE_LARGEFILE(readdir,readdir64)
+#define readdir_r_or_readdir64_r        CHOOSE_LARGEFILE(readdir_r,readdir64_r)
+#define stat_or_stat64                  CHOOSE_LARGEFILE(stat,stat64)
+#define truncate_or_truncate64          CHOOSE_LARGEFILE(truncate,truncate64)
+#define scm_from_off_t_or_off64_t       CHOOSE_LARGEFILE(scm_from_off_t,scm_from_int64)
+#define scm_from_ino_t_or_ino64_t       CHOOSE_LARGEFILE(scm_from_ulong,scm_from_uint64)
+#define scm_from_blkcnt_t_or_blkcnt64_t CHOOSE_LARGEFILE(scm_from_ulong,scm_from_uint64)
+#define scm_to_off_t_or_off64_t         CHOOSE_LARGEFILE(scm_to_off_t,scm_to_int64)
+
+#if SIZEOF_OFF_T == 4
+#  define scm_to_off_t    scm_to_int32
+#  define scm_from_off_t  scm_from_int32
+#elif SIZEOF_OFF_T == 8
+#  define scm_to_off_t    scm_to_int64
+#  define scm_from_off_t  scm_from_int64
+#else
+#  error sizeof(off_t) is not 4 or 8.
+#endif
+#define scm_to_off64_t    scm_to_int64
+#define scm_from_off64_t  scm_from_int64
+
+
+#endif  /* SCM__SCM_H */
+
 /*
   Local Variables:
   c-file-style: "gnu"
