@@ -1,4 +1,4 @@
-/* Copyright (C) 1996,1997,2000,2001, 2004 Free Software Foundation, Inc.
+/* Copyright (C) 1996,1997,2000,2001, 2004, 2006 Free Software Foundation, Inc.
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -12,7 +12,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <stdio.h>
@@ -65,6 +65,7 @@ static scm_t_bits tc16_fluid;
 #define IS_FLUID(x)         SCM_SMOB_PREDICATE(tc16_fluid, (x))
 #define FLUID_NUM(x)        ((size_t)SCM_SMOB_DATA(x))
 #define FLUID_NEXT(x)       SCM_SMOB_OBJECT_2(x)
+#define FLUID_NEXT_LOC(x)       SCM_SMOB_OBJECT_2_LOC(x)
 #define SET_FLUID_NEXT(x,y) SCM_SET_SMOB_OBJECT_2((x), (y))
 
 static scm_t_bits tc16_dynamic_state;
@@ -73,6 +74,7 @@ static scm_t_bits tc16_dynamic_state;
 #define DYNAMIC_STATE_FLUIDS(x)        SCM_SMOB_OBJECT(x)
 #define SET_DYNAMIC_STATE_FLUIDS(x, y) SCM_SET_SMOB_OBJECT((x), (y))
 #define DYNAMIC_STATE_NEXT(x)          SCM_SMOB_OBJECT_2(x)
+#define DYNAMIC_STATE_NEXT_LOC(x)          SCM_SMOB_OBJECT_2_LOC(x)
 #define SET_DYNAMIC_STATE_NEXT(x, y)   SCM_SET_SMOB_OBJECT_2((x), (y))
 
 /* Weak lists of all dynamic states and all fluids.
@@ -145,7 +147,7 @@ scan_dynamic_states_and_fluids (void *dummy1 SCM_UNUSED,
 	  *fluidp = FLUID_NEXT (*fluidp);
 	}
       else
-	fluidp = &FLUID_NEXT (*fluidp);
+	fluidp = FLUID_NEXT_LOC (*fluidp);
     }
 
   /* Scan all dynamic states and remove the unmarked ones.  The live
@@ -166,7 +168,7 @@ scan_dynamic_states_and_fluids (void *dummy1 SCM_UNUSED,
 	    if (allocated_fluids[i] == 0)
 	      SCM_SIMPLE_VECTOR_SET (fluids, i, SCM_BOOL_F);
 
-	  statep = &DYNAMIC_STATE_NEXT (*statep);
+	  statep = DYNAMIC_STATE_NEXT_LOC (*statep);
 	}
     }
 
@@ -197,10 +199,11 @@ next_fluid_num ()
 {
   size_t n;
 
-  scm_frame_begin (0);
-  scm_i_frame_pthread_mutex_lock (&fluid_admin_mutex);
+  scm_dynwind_begin (0);
+  scm_i_dynwind_pthread_mutex_lock (&fluid_admin_mutex);
 
-  if (allocated_fluids_num == allocated_fluids_len)
+  if ((allocated_fluids_len > 0) &&
+      (allocated_fluids_num == allocated_fluids_len))
     {
       /* All fluid numbers are in use.  Run a GC to try to free some
 	 up.
@@ -245,7 +248,7 @@ next_fluid_num ()
   allocated_fluids_num += 1;
   allocated_fluids[n] = 1;
   
-  scm_frame_end ();
+  scm_dynwind_end ();
   return n;
 }
 
@@ -417,13 +420,13 @@ scm_c_with_fluids (SCM fluids, SCM values, SCM (*cproc) (), void *cdata)
 			     cproc, cdata);
   
   data = scm_cons (fluids, values);
-  scm_frame_begin (SCM_F_FRAME_REWINDABLE);
-  scm_frame_rewind_handler_with_scm (swap_fluids, data,
+  scm_dynwind_begin (SCM_F_DYNWIND_REWINDABLE);
+  scm_dynwind_rewind_handler_with_scm (swap_fluids, data,
 				     SCM_F_WIND_EXPLICITLY);
-  scm_frame_unwind_handler_with_scm (swap_fluids_reverse, data,
+  scm_dynwind_unwind_handler_with_scm (swap_fluids_reverse, data,
 				     SCM_F_WIND_EXPLICITLY);
   ans = cproc (cdata);
-  scm_frame_end ();
+  scm_dynwind_end ();
   return ans;
 }
 #undef FUNC_NAME
@@ -445,10 +448,10 @@ scm_c_with_fluid (SCM fluid, SCM value, SCM (*cproc) (), void *cdata)
 {
   SCM ans;
 
-  scm_frame_begin (SCM_F_FRAME_REWINDABLE);
-  scm_frame_fluid (fluid, value);
+  scm_dynwind_begin (SCM_F_DYNWIND_REWINDABLE);
+  scm_dynwind_fluid (fluid, value);
   ans = cproc (cdata);
-  scm_frame_end ();
+  scm_dynwind_end ();
   return ans;
 }
 #undef FUNC_NAME
@@ -463,11 +466,11 @@ swap_fluid (SCM data)
 }
 
 void
-scm_frame_fluid (SCM fluid, SCM value)
+scm_dynwind_fluid (SCM fluid, SCM value)
 {
   SCM data = scm_cons (fluid, value);
-  scm_frame_rewind_handler_with_scm (swap_fluid, data, SCM_F_WIND_EXPLICITLY);
-  scm_frame_unwind_handler_with_scm (swap_fluid, data, SCM_F_WIND_EXPLICITLY);
+  scm_dynwind_rewind_handler_with_scm (swap_fluid, data, SCM_F_WIND_EXPLICITLY);
+  scm_dynwind_unwind_handler_with_scm (swap_fluid, data, SCM_F_WIND_EXPLICITLY);
 }
 
 SCM
@@ -555,13 +558,13 @@ swap_dynamic_state (SCM loc)
 }
 
 void
-scm_frame_current_dynamic_state (SCM state)
+scm_dynwind_current_dynamic_state (SCM state)
 {
   SCM loc = scm_cons (state, SCM_EOL);
   scm_assert_smob_type (tc16_dynamic_state, state);
-  scm_frame_rewind_handler_with_scm (swap_dynamic_state, loc,
+  scm_dynwind_rewind_handler_with_scm (swap_dynamic_state, loc,
 				     SCM_F_WIND_EXPLICITLY);
-  scm_frame_unwind_handler_with_scm (swap_dynamic_state, loc,
+  scm_dynwind_unwind_handler_with_scm (swap_dynamic_state, loc,
 				     SCM_F_WIND_EXPLICITLY);
 }
 
@@ -569,10 +572,10 @@ void *
 scm_c_with_dynamic_state (SCM state, void *(*func)(void *), void *data)
 {
   void *result;
-  scm_frame_begin (SCM_F_FRAME_REWINDABLE);
-  scm_frame_current_dynamic_state (state);
+  scm_dynwind_begin (SCM_F_DYNWIND_REWINDABLE);
+  scm_dynwind_current_dynamic_state (state);
   result = func (data);
-  scm_frame_end ();
+  scm_dynwind_end ();
   return result;
 }
 
@@ -583,10 +586,10 @@ SCM_DEFINE (scm_with_dynamic_state, "with-dynamic-state", 2, 0, 0,
 #define FUNC_NAME s_scm_with_dynamic_state
 {
   SCM result;
-  scm_frame_begin (SCM_F_FRAME_REWINDABLE);
-  scm_frame_current_dynamic_state (state);
+  scm_dynwind_begin (SCM_F_DYNWIND_REWINDABLE);
+  scm_dynwind_current_dynamic_state (state);
   result = scm_call_0 (proc);
-  scm_frame_end ();
+  scm_dynwind_end ();
   return result;
 }
 #undef FUNC_NAME
