@@ -19,16 +19,20 @@
 
 /* Headers.  */
 
+#define _LARGEFILE64_SOURCE      /* ask for stat64 etc */
+
 #if HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
 #include <stdio.h>
 #include <errno.h>
+#include <fcntl.h>  /* for chsize on mingw */
 
 #include "libguile/_scm.h"
 #include "libguile/async.h"
 #include "libguile/eval.h"
+#include "libguile/fports.h"  /* direct access for seek and truncate */
 #include "libguile/objects.h"
 #include "libguile/goops.h"
 #include "libguile/smob.h"
@@ -64,9 +68,17 @@
 #include <sys/ioctl.h>
 #endif
 
-#ifdef __MINGW32__
-#include <fcntl.h>
+/* Mingw (version 3.4.5, circa 2006) has ftruncate as an alias for chsize
+   already, but have this code here in case that wasn't so in past versions,
+   or perhaps to help other minimal DOS environments.
+
+   gnulib ftruncate.c has code using fcntl F_CHSIZE and F_FREESP, which
+   might be possibilities if we've got other systems without ftruncate.  */
+
+#if HAVE_CHSIZE && ! HAVE_FTRUNCATE
 #define ftruncate(fd, size) chsize (fd, size)
+#undef HAVE_FTRUNCATE
+#define HAVE_FTRUNCATE 1
 #endif
 
 
@@ -163,50 +175,50 @@ scm_make_port_type (char *name,
 }
 
 void
-scm_set_port_mark (long tc, SCM (*mark) (SCM))
+scm_set_port_mark (scm_t_bits tc, SCM (*mark) (SCM))
 {
   scm_ptobs[SCM_TC2PTOBNUM (tc)].mark = mark;
 }
 
 void
-scm_set_port_free (long tc, size_t (*free) (SCM))
+scm_set_port_free (scm_t_bits tc, size_t (*free) (SCM))
 {
   scm_ptobs[SCM_TC2PTOBNUM (tc)].free = free;
 }
 
 void
-scm_set_port_print (long tc, int (*print) (SCM exp, SCM port,
+scm_set_port_print (scm_t_bits tc, int (*print) (SCM exp, SCM port,
 					   scm_print_state *pstate))
 {
   scm_ptobs[SCM_TC2PTOBNUM (tc)].print = print;
 }
 
 void
-scm_set_port_equalp (long tc, SCM (*equalp) (SCM, SCM))
+scm_set_port_equalp (scm_t_bits tc, SCM (*equalp) (SCM, SCM))
 {
   scm_ptobs[SCM_TC2PTOBNUM (tc)].equalp = equalp;
 }
 
 void
-scm_set_port_flush (long tc, void (*flush) (SCM port))
+scm_set_port_flush (scm_t_bits tc, void (*flush) (SCM port))
 {
    scm_ptobs[SCM_TC2PTOBNUM (tc)].flush = flush;
 }
 
 void
-scm_set_port_end_input (long tc, void (*end_input) (SCM port, int offset))
+scm_set_port_end_input (scm_t_bits tc, void (*end_input) (SCM port, int offset))
 {
   scm_ptobs[SCM_TC2PTOBNUM (tc)].end_input = end_input;
 }
 
 void
-scm_set_port_close (long tc, int (*close) (SCM))
+scm_set_port_close (scm_t_bits tc, int (*close) (SCM))
 {
   scm_ptobs[SCM_TC2PTOBNUM (tc)].close = close;
 }
 
 void
-scm_set_port_seek (long tc, off_t (*seek) (SCM port,
+scm_set_port_seek (scm_t_bits tc, off_t (*seek) (SCM port,
 					   off_t OFFSET,
 					   int WHENCE))
 {
@@ -214,13 +226,13 @@ scm_set_port_seek (long tc, off_t (*seek) (SCM port,
 }
 
 void
-scm_set_port_truncate (long tc, void (*truncate) (SCM port, off_t length))
+scm_set_port_truncate (scm_t_bits tc, void (*truncate) (SCM port, off_t length))
 {
   scm_ptobs[SCM_TC2PTOBNUM (tc)].truncate = truncate;
 }
 
 void
-scm_set_port_input_waiting (long tc, int (*input_waiting) (SCM))
+scm_set_port_input_waiting (scm_t_bits tc, int (*input_waiting) (SCM))
 {
   scm_ptobs[SCM_TC2PTOBNUM (tc)].input_waiting = input_waiting;
 }
@@ -815,6 +827,8 @@ scm_c_port_for_each (void (*proc)(void *data, SCM p), void *data)
 
   for (i = 0; i < n; i++)
     proc (data, SCM_SIMPLE_VECTOR_REF (ports, i));
+
+  scm_remember_upto_here_1 (ports);
 }
 
 SCM_DEFINE (scm_port_for_each, "port-for-each", 1, 0, 0,
@@ -1370,68 +1384,95 @@ SCM_DEFINE (scm_seek, "seek", 3, 0, 0,
 	    "@end lisp")
 #define FUNC_NAME s_scm_seek
 {
-  off_t off;
-  off_t rv;
   int how;
 
   fd_port = SCM_COERCE_OUTPORT (fd_port);
 
-  if (sizeof (off_t) == sizeof (scm_t_intmax))
-    off = scm_to_intmax (offset);
-  else
-    off = scm_to_long (offset);
   how = scm_to_int (whence);
-
   if (how != SEEK_SET && how != SEEK_CUR && how != SEEK_END)
     SCM_OUT_OF_RANGE (3, whence);
-  if (SCM_OPPORTP (fd_port))
+
+  if (SCM_OPFPORTP (fd_port))
+    {
+      /* go direct to fport code to allow 64-bit offsets */
+      return scm_i_fport_seek (fd_port, offset, how);
+    }
+  else if (SCM_OPPORTP (fd_port))
     {
       scm_t_ptob_descriptor *ptob = scm_ptobs + SCM_PTOBNUM (fd_port);
+      off_t off = scm_to_off_t (offset);
+      off_t rv;
 
       if (!ptob->seek)
 	SCM_MISC_ERROR ("port is not seekable", 
                         scm_cons (fd_port, SCM_EOL));
       else
 	rv = ptob->seek (fd_port, off, how);
+      return scm_from_off_t (rv);
     }
   else /* file descriptor?.  */
     {
-      rv = lseek (scm_to_int (fd_port), off, how);
+      off_t_or_off64_t off = scm_to_off_t_or_off64_t (offset);
+      off_t_or_off64_t rv;
+      rv = lseek_or_lseek64 (scm_to_int (fd_port), off, how);
       if (rv == -1)
 	SCM_SYSERROR;
+      return scm_from_off_t_or_off64_t (rv);
     }
-  return scm_from_intmax (rv);
 }
 #undef FUNC_NAME
 
-#ifdef __MINGW32__
-/* Define this function since it is not supported under Windows. */
-static int truncate (char *file, int length)
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
+
+/* Mingw has ftruncate(), perhaps implemented above using chsize, but
+   doesn't have the filename version truncate(), hence this code.  */
+#if HAVE_FTRUNCATE && ! HAVE_TRUNCATE
+static int
+truncate (const char *file, off_t length)
 {
-  int ret = -1, fdes;
-  if ((fdes = open (file, O_BINARY | O_WRONLY)) != -1)
+  int ret, fdes;
+
+  fdes = open (file, O_BINARY | O_WRONLY);
+  if (fdes == -1)
+    return -1;
+
+  ret = ftruncate (fdes, length);
+  if (ret == -1)
     {
-      ret = chsize (fdes, length);
+      int save_errno = errno;
       close (fdes);
+      errno = save_errno;
+      return -1;
     }
-  return ret;
+
+  return close (fdes);
 }
-#endif /* __MINGW32__ */
+#endif /* HAVE_FTRUNCATE && ! HAVE_TRUNCATE */
 
 SCM_DEFINE (scm_truncate_file, "truncate-file", 1, 1, 0,
             (SCM object, SCM length),
-	    "Truncates the object referred to by @var{object} to at most\n"
-	    "@var{length} bytes.  @var{object} can be a string containing a\n"
-	    "file name or an integer file descriptor or a port.\n"
-	    "@var{length} may be omitted if @var{object} is not a file name,\n"
-	    "in which case the truncation occurs at the current port\n"
-	    "position.  The return value is unspecified.")
+	    "Truncate @var{file} to @var{length} bytes.  @var{file} can be a\n"
+	    "filename string, a port object, or an integer file descriptor.\n"
+	    "The return value is unspecified.\n"
+	    "\n"
+	    "For a port or file descriptor @var{length} can be omitted, in\n"
+	    "which case the file is truncated at the current position (per\n"
+	    "@code{ftell} above).\n"
+	    "\n"
+	    "On most systems a file can be extended by giving a length\n"
+	    "greater than the current size, but this is not mandatory in the\n"
+	    "POSIX standard.")
 #define FUNC_NAME s_scm_truncate_file
 {
   int rv;
-  off_t c_length;
 
-  /* object can be a port, fdes or filename.  */
+  /* "object" can be a port, fdes or filename.
+
+     Negative "length" makes no sense, but it's left to truncate() or
+     ftruncate() to give back an error for that (normally EINVAL).
+     */
 
   if (SCM_UNBNDP (length))
     {
@@ -1441,15 +1482,22 @@ SCM_DEFINE (scm_truncate_file, "truncate-file", 1, 1, 0,
       
       length = scm_seek (object, SCM_INUM0, scm_from_int (SEEK_CUR));
     }
-  c_length = scm_to_size_t (length);
 
   object = SCM_COERCE_OUTPORT (object);
   if (scm_is_integer (object))
     {
-      SCM_SYSCALL (rv = ftruncate (scm_to_int (object), c_length));
+      off_t_or_off64_t c_length = scm_to_off_t_or_off64_t (length);
+      SCM_SYSCALL (rv = ftruncate_or_ftruncate64 (scm_to_int (object),
+                                                  c_length));
+    }
+  else if (SCM_OPOUTFPORTP (object))
+    {
+      /* go direct to fport code to allow 64-bit offsets */
+      rv = scm_i_fport_truncate (object, length);
     }
   else if (SCM_OPOUTPORTP (object))
     {
+      off_t c_length = scm_to_off_t (length);
       scm_t_port *pt = SCM_PTAB_ENTRY (object);
       scm_t_ptob_descriptor *ptob = scm_ptobs + SCM_PTOBNUM (object);
       
@@ -1465,9 +1513,10 @@ SCM_DEFINE (scm_truncate_file, "truncate-file", 1, 1, 0,
     }
   else
     {
+      off_t_or_off64_t c_length = scm_to_off_t_or_off64_t (length);
       char *str = scm_to_locale_string (object);
       int eno;
-      SCM_SYSCALL (rv = truncate (str, c_length));
+      SCM_SYSCALL (rv = truncate_or_truncate64 (str, c_length));
       eno = errno;
       free (str);
       errno = eno;
