@@ -1,4 +1,4 @@
-/* Copyright (C) 1995,1996,1997,1998,1999,2000,2001, 2003, 2004, 2005 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1997,1998,1999,2000,2001, 2003, 2004, 2005, 2006 Free Software Foundation, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -12,7 +12,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 
@@ -497,8 +497,10 @@ bdtime2c (SCM sbd_time, struct tm *lt, int pos, const char *subr)
   lt->tm_wday = scm_to_int (SCM_SIMPLE_VECTOR_REF (sbd_time, 6));
   lt->tm_yday = scm_to_int (SCM_SIMPLE_VECTOR_REF (sbd_time, 7));
   lt->tm_isdst = scm_to_int (SCM_SIMPLE_VECTOR_REF (sbd_time, 8));
+#if HAVE_STRUCT_TM_TM_GMTOFF
+  lt->tm_gmtoff = - scm_to_int (SCM_SIMPLE_VECTOR_REF (sbd_time, 9));
+#endif
 #ifdef HAVE_TM_ZONE
-  lt->tm_gmtoff = scm_to_int (SCM_SIMPLE_VECTOR_REF (sbd_time, 9));
   if (scm_is_false (SCM_SIMPLE_VECTOR_REF (sbd_time, 10)))
     lt->tm_zone = NULL;
   else
@@ -525,14 +527,15 @@ SCM_DEFINE (scm_mktime, "mktime", 1, 1, 0,
   char **oldenv;
   int err;
 
-  scm_frame_begin (0);
+  scm_dynwind_begin (0);
 
   bdtime2c (sbd_time, &lt, SCM_ARG1, FUNC_NAME);
 #if HAVE_STRUCT_TM_TM_ZONE
-  scm_frame_free ((char *)lt.tm_zone);
+  scm_dynwind_free ((char *)lt.tm_zone);
 #endif
 
-  SCM_CRITICAL_SECTION_START;
+  scm_dynwind_critical_section (SCM_BOOL_F);
+
   oldenv = setzone (zone, SCM_ARG2, FUNC_NAME);
 #ifdef LOCALTIME_CACHE
   tzset ();
@@ -585,11 +588,10 @@ SCM_DEFINE (scm_mktime, "mktime", 1, 1, 0,
 
   result = scm_cons (scm_from_long (itime),
 		     filltime (&lt, zoff, zname));
-  SCM_CRITICAL_SECTION_END;
   if (zname)
     free (zname);
 
-  scm_frame_end ();
+  scm_dynwind_end ();
   return result;
 }
 #undef FUNC_NAME
@@ -611,13 +613,22 @@ SCM_DEFINE (scm_tzset, "tzset", 0, 0, 0,
 
 SCM_DEFINE (scm_strftime, "strftime", 2, 0, 0,
             (SCM format, SCM stime),
-	    "Formats a time specification @var{time} using @var{template}.  @var{time}\n"
-	    "is an object with time components in the form returned by @code{localtime}\n"
-	    "or @code{gmtime}.  @var{template} is a string which can include formatting\n"
-	    "specifications introduced by a @code{%} character.  The formatting of\n"
-	    "month and day names is dependent on the current locale.  The value returned\n"
-	    "is the formatted string.\n"
-	    "@xref{Formatting Date and Time, , , libc, The GNU C Library Reference Manual}.)")
+	    "Return a string which is broken-down time structure @var{stime}\n"
+	    "formatted according to the given @var{format} string.\n"
+	    "\n"
+	    "@var{format} contains field specifications introduced by a\n"
+	    "@samp{%} character.  See @ref{Formatting Calendar Time,,, libc,\n"
+	    "The GNU C Library Reference Manual}, or @samp{man 3 strftime},\n"
+	    "for the available formatting.\n"
+	    "\n"
+	    "@lisp\n"
+	    "(strftime \"%c\" (localtime (current-time)))\n"
+	    "@result{} \"Mon Mar 11 20:17:43 2002\"\n"
+	    "@end lisp\n"
+	    "\n"
+	    "If @code{setlocale} has been called (@pxref{Locales}), month\n"
+	    "and day names are from the current locale and in the locale\n"
+	    "character set.")
 #define FUNC_NAME s_scm_strftime
 {
   struct tm t;
@@ -653,19 +664,18 @@ SCM_DEFINE (scm_strftime, "strftime", 2, 0, 0,
        environment.  interrupts and thread switching must be deferred
        until TZ is restored.  */
     char **oldenv = NULL;
-    SCM *velts = (SCM *) SCM_VELTS (stime);
+    SCM zone_spec = SCM_SIMPLE_VECTOR_REF (stime, 10);
     int have_zone = 0;
 
-    if (scm_is_true (velts[10]) && *SCM_STRING_CHARS (velts[10]) != 0)
+    if (scm_is_true (zone_spec) && scm_c_string_length (zone_spec) > 0)
       {
 	/* it's not required that the TZ setting be correct, just that
 	   it has the right name.  so try something like TZ=EST0.
 	   using only TZ=EST would be simpler but it doesn't work on
 	   some OSs, e.g., Solaris.  */
 	SCM zone =
-	  scm_string_append (scm_cons (velts[10],
-				       scm_cons (scm_from_locale_string ("0"),
-						 SCM_EOL)));
+	  scm_string_append (scm_list_2 (zone_spec,
+					 scm_from_locale_string ("0")));
 
 	have_zone = 1;
 	SCM_CRITICAL_SECTION_START;
@@ -690,7 +700,7 @@ SCM_DEFINE (scm_strftime, "strftime", 2, 0, 0,
 #if !defined (HAVE_TM_ZONE)
     if (have_zone)
       {
-	restorezone (velts[10], oldenv, FUNC_NAME);
+	restorezone (zone_spec, oldenv, FUNC_NAME);
 	SCM_CRITICAL_SECTION_END;
       }
 #endif
@@ -723,6 +733,7 @@ SCM_DEFINE (scm_strptime, "strptime", 2, 0, 0,
 {
   struct tm t;
   const char *fmt, *str, *rest;
+  long zoff;
 
   SCM_VALIDATE_STRING (1, format);
   SCM_VALIDATE_STRING (2, string);
@@ -740,6 +751,9 @@ SCM_DEFINE (scm_strptime, "strptime", 2, 0, 0,
   tm_init (tm_year);
   tm_init (tm_wday);
   tm_init (tm_yday);
+#if HAVE_STRUCT_TM_TM_GMTOFF
+  tm_init (tm_gmtoff);
+#endif
 #undef tm_init
 
   /* GNU glibc strptime() "%s" is affected by the current timezone, since it
@@ -758,7 +772,15 @@ SCM_DEFINE (scm_strptime, "strptime", 2, 0, 0,
       SCM_SYSERROR;
     }
 
-  return scm_cons (filltime (&t, 0, NULL),
+  /* tm_gmtoff is set by GNU glibc strptime "%s", so capture it when
+     available */
+#if HAVE_STRUCT_TM_TM_GMTOFF
+  zoff = - t.tm_gmtoff;  /* seconds west, not east */
+#else
+  zoff = 0;
+#endif
+
+  return scm_cons (filltime (&t, zoff, NULL),
 		   scm_from_signed_integer (rest - str));
 }
 #undef FUNC_NAME
