@@ -1,4 +1,4 @@
-/* Copyright (C) 1995,1996,1997,1998,2000,2001,2002,2003,2004, 2005 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1997,1998,2000,2001,2002,2003,2004, 2005, 2006 Free Software Foundation, Inc.
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -12,7 +12,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 
@@ -368,9 +368,9 @@ string_set (scm_t_array_handle *h, ssize_t pos, SCM val)
 {
   pos += h->base;
   if (SCM_I_ARRAYP (h->array))
-    return scm_c_string_set_x (SCM_I_ARRAY_V (h->array), pos, val);
+    scm_c_string_set_x (SCM_I_ARRAY_V (h->array), pos, val);
   else
-    return scm_c_string_set_x (h->array, pos, val);
+    scm_c_string_set_x (h->array, pos, val);
 }
 
 static void
@@ -380,9 +380,9 @@ bitvector_set (scm_t_array_handle *h, ssize_t pos, SCM val)
   pos += scm_array_handle_bit_elements_offset (h);
   mask = 1l << (pos % 32);
   if (scm_to_bool (val))
-    ((scm_t_uint32 *)h->elements)[pos/32] |= mask;
+    ((scm_t_uint32 *)h->writable_elements)[pos/32] |= mask;
   else
-    ((scm_t_uint32 *)h->elements)[pos/32] &= ~mask;
+    ((scm_t_uint32 *)h->writable_elements)[pos/32] &= ~mask;
 }
 
 static void
@@ -796,6 +796,18 @@ SCM_DEFINE (scm_dimensions_to_uniform_array, "dimensions->uniform-array", 2, 1,
 
   if (scm_is_integer (dims))
     dims = scm_list_1 (dims);
+
+  if (SCM_UNBNDP (fill))
+    {
+      /* Using #\nul as the prototype yields a s8 array, but numeric
+	 arrays can't store characters, so we have to special case this.
+      */
+      if (scm_is_eq (prot, SCM_MAKE_CHAR (0)))
+	fill = scm_from_int (0);
+      else
+	fill = prot;
+    }
+
   return scm_make_typed_array (prototype_to_type (prot), fill, dims);
 }
 #undef FUNC_NAME
@@ -849,7 +861,7 @@ SCM_DEFINE (scm_make_shared_array, "make-shared-array", 2, 0, 1,
   SCM imap;
   size_t k;
   ssize_t i;
-  long old_min, new_min, old_max, new_max;
+  long old_base, old_min, new_min, old_max, new_max;
   scm_t_array_dim *s;
 
   SCM_VALIDATE_REST_ARGUMENT (dims);
@@ -861,7 +873,7 @@ SCM_DEFINE (scm_make_shared_array, "make-shared-array", 2, 0, 1,
   if (SCM_I_ARRAYP (oldra))
     {
       SCM_I_ARRAY_V (ra) = SCM_I_ARRAY_V (oldra);
-      old_min = old_max = SCM_I_ARRAY_BASE (oldra);
+      old_base = old_min = old_max = SCM_I_ARRAY_BASE (oldra);
       s = scm_array_handle_dims (&old_handle);
       k = scm_array_handle_rank (&old_handle);
       while (k--)
@@ -875,7 +887,7 @@ SCM_DEFINE (scm_make_shared_array, "make-shared-array", 2, 0, 1,
   else
     {
       SCM_I_ARRAY_V (ra) = oldra;
-      old_min = 0;
+      old_base = old_min = 0;
       old_max = scm_c_generalized_vector_length (oldra) - 1;
     }
 
@@ -897,7 +909,7 @@ SCM_DEFINE (scm_make_shared_array, "make-shared-array", 2, 0, 1,
 
   imap = scm_apply_0 (mapfunc, scm_reverse (inds));
   i = scm_array_handle_pos (&old_handle, imap);
-  SCM_I_ARRAY_BASE (ra) = new_min = new_max = i;
+  SCM_I_ARRAY_BASE (ra) = new_min = new_max = i + old_base;
   indptr = inds;
   k = SCM_I_ARRAY_NDIM (ra);
   while (k--)
@@ -1136,18 +1148,7 @@ SCM_DEFINE (scm_array_in_bounds_p, "array-in-bounds?", 1, 0, 1,
 
   SCM_VALIDATE_REST_ARGUMENT (args);
 
-  if (scm_is_generalized_vector (v))
-    {
-      long ind;
-
-      if (!scm_is_pair (args))
-	SCM_WRONG_NUM_ARGS ();
-      ind = scm_to_long (SCM_CAR (args));
-      args = SCM_CDR (args);
-      res = scm_from_bool (ind >= 0
-			   && ind < scm_c_generalized_vector_length (v));
-    }
-  else if (SCM_I_ARRAYP (v) || SCM_I_ENCLOSED_ARRAYP (v))
+  if (SCM_I_ARRAYP (v) || SCM_I_ENCLOSED_ARRAYP (v))
     {
       size_t k = SCM_I_ARRAY_NDIM (v);
       scm_t_array_dim *s = SCM_I_ARRAY_DIMS (v);
@@ -1172,6 +1173,21 @@ SCM_DEFINE (scm_array_in_bounds_p, "array-in-bounds?", 1, 0, 1,
 	    }
 	}
     }
+  else if (scm_is_generalized_vector (v))
+    {
+      /* Since real arrays have been covered above, all generalized
+	 vectors are guaranteed to be zero-origin here.
+      */
+
+      long ind;
+
+      if (!scm_is_pair (args))
+	SCM_WRONG_NUM_ARGS ();
+      ind = scm_to_long (SCM_CAR (args));
+      args = SCM_CDR (args);
+      res = scm_from_bool (ind >= 0
+			   && ind < scm_c_generalized_vector_length (v));
+    }
   else
     scm_wrong_type_arg_msg (NULL, 0, v, "array");
 
@@ -2660,7 +2676,7 @@ read_decimal_integer (SCM port, int c, ssize_t *resp)
 SCM
 scm_i_read_array (SCM port, int c)
 {
-  size_t rank;
+  ssize_t rank;
   int got_rank;
   char tag[80];
   int tag_len;
@@ -2756,7 +2772,7 @@ scm_i_read_array (SCM port, int c)
   elements = scm_read (port);
 
   if (scm_is_false (shape))
-    shape = scm_from_size_t (rank);
+    shape = scm_from_ssize_t (rank);
   else if (scm_ilength (shape) != rank)
     scm_i_input_error 
       (NULL, port,
@@ -2885,7 +2901,7 @@ scm_aind (SCM ra, SCM args, const char *what)
     args = scm_list_1 (args);
   
   scm_array_get_handle (ra, &handle);
-  pos = scm_array_handle_pos (&handle, args);
+  pos = scm_array_handle_pos (&handle, args) + SCM_I_ARRAY_BASE (ra);
   scm_array_handle_release (&handle);
   return pos;
 }
